/**
   Basic test of mpu6050
   Based on https://www.hackster.io/theSTEMpedia/diy-accelerometer-based-controller-1b32db

   Outputs movements to Serial output (Serial Monitor) and to display, can for instance be the base of a game controller.
   If texts are received from Serial connection, they are echod.

   Technical data of the mpu6050 sensor:
   MPU-6000 and MPU-6050 Register Map and Descriptions Revision 4.2, p.40*
   https://invensense.tdk.com/wp-content/uploads/2015/02/MPU-6000-Register-Map1.pdf

   Uses U8g2 library (for display only). Uses basic i2c Wire communication with the sensor.
*/

#include <Wire.h>
#include <U8g2lib.h>

// display setup:
U8X8_SSD1306_128X64_NONAME_HW_I2C display(U8X8_PIN_NONE);

const int MPU_ADDR = 0x68;                              // I2C address of the MPU-6050. If AD0 pin is set to HIGH, the I2C address will be 0x69.
int accelerometer_x, accelerometer_y, accelerometer_z; // variables for accelerometer raw data
int gyro_x, gyro_y, gyro_z;                           // variables for gyro raw data
int temperature;                                     // variables for temperature data

#define THRESHOLD 2000

// variables used as text buffers for input received:
String line;
char buf[16];

void setup() {
  Serial.begin(9600);

  // setup I2C communication with the sensor:
  Wire.begin();
  Wire.beginTransmission(MPU_ADDR); // Begins a transmission to the I2C slave (GY-521 board)
  Wire.write(0x6B);                 // write to PWR_MGMT_1 register
  Wire.write(0);                    // set to zero (wakes up the MPU-6050)
  Wire.endTransmission(true);

  // initialize display:
  display.begin();
  display.setPowerSave(0);
  display.setFont(u8x8_font_pxplusibmcgathin_f);
  display.drawString(0, 0, "ACCL/GYRO");
}

void loop() {
  // initiate i2c request:
  Wire.beginTransmission(MPU_ADDR);
  Wire.write(0x3B);                        // starting with register 0x3B (ACCEL_XOUT_H)**
  Wire.endTransmission(false);             // the parameter indicates that the Arduino will send a restart. As a result, the connection is kept active.
  Wire.requestFrom(MPU_ADDR, 7 * 2, true); // request a total of 7*2=14 registers

  // read data from the registers of the sensor:
  // "Wire.read()<<8 | Wire.read();" means two registers are read and stored in the same variable
  accelerometer_x = Wire.read() << 8 | Wire.read();   // reading registers: 0x3B (ACCEL_XOUT_H) and 0x3C (ACCEL_XOUT_L)
  accelerometer_y = Wire.read() << 8 | Wire.read();   // reading registers: 0x3D (ACCEL_YOUT_H) and 0x3E (ACCEL_YOUT_L)
  accelerometer_z = Wire.read() << 8 | Wire.read();   // reading registers: 0x3F (ACCEL_ZOUT_H) and 0x40 (ACCEL_ZOUT_L)
  temperature = Wire.read() << 8 | Wire.read();       // reading registers: 0x41 (TEMP_OUT_H) and 0x42 (TEMP_OUT_L)
  gyro_x = Wire.read() << 8 | Wire.read();            // reading registers: 0x43 (GYRO_XOUT_H) and 0x44 (GYRO_XOUT_L)
  gyro_y = Wire.read() << 8 | Wire.read();            // reading registers: 0x45 (GYRO_YOUT_H) and 0x46 (GYRO_YOUT_L)
  gyro_z = Wire.read() << 8 | Wire.read();            // reading registers: 0x47 (GYRO_ZOUT_H) and 0x48 (GYRO_ZOUT_L)

  /*  Serial.print(accelerometer_x);
    Serial.print(",");
    Serial.print(accelerometer_y);
    Serial.print(",");
    Serial.print(accelerometer_z);
    Serial.println("");
  */
  // 'translate' the accelerometer data into basic movements (directions) and print those:
  if (accelerometer_x > THRESHOLD) message("forward");
  else if (accelerometer_y > THRESHOLD) message("left");
  else {
    accelerometer_y = -(accelerometer_y);
    accelerometer_x = -(accelerometer_x);

    if (accelerometer_x > THRESHOLD) message("backward");
    else if (accelerometer_y > THRESHOLD) message("right");
    else if (accelerometer_z > THRESHOLD) message("stop");
    else message("-");
  }

  delay(50);

  // receive data from Serial connection:
  while (Serial.available()>0) {
       char readchar = Serial.read();
       if (readchar>=32 && readchar<127) line += readchar; // if valid character add it to line (ignore/drop invalids chars)
       delay(2);
  }
  if (line.length() > 1) { // did we receive anything?
    // Print text we received on the display:

    line = ""; // make line empty to prevent processing the same line twice
  }
  delay(50);
}


/*
 * Show a message on line 1 of the display and also show it on Serial output.
 */
void message(char* message) {
  display.clearLine(1); // clear line 1 of the display
  display.drawString(0,1,message); // print message on line 1 of the display
  Serial.println(message);  // print message to serial connection
}

/*
 * Get first part of String s
 */
String getPart1(String s) {
  int space = s.lastIndexOf(' '); // is there a space? (' ')
  if (space>0) return s.substring(0, space); // return part until the space
  return s; // if there was no space, just return the whole String
}

/*
 * Get second part of String s
 */
String getPart2(String s) {
  int space = s.lastIndexOf(' '); // is there a space? (' ')
  if (space>0) return s.substring(space+1); // return part after the space
  return ""; // if there was no space, return empty String
}
